import { createJoinFunction } from "better-path-join/index.mjs";
import { FileTreeRemovalFailure, MultipleFailures, Success } from "@ts-schema-autogen/status/index.mjs";
import { listSymbolInstruction } from "./instruction.mjs";
import { ensureOutputDescriptorArray } from "./output-descriptor.mjs";
import { ConfigLoader } from "./load-config.mjs";
/** Determine output files from instruction and delete them */

export function cleanUnit(param) {
  const {
    directory
  } = param;
  const {
    remove
  } = param.fsx;
  const join = createJoinFunction(param.path);

  function handleOutputDescriptor(desc) {
    const filename = join(directory, desc.filename);
    const promise = remove(filename);
    return {
      filename,
      promise
    };
  }

  const result = listSymbolInstruction(param.instruction).map(instruction => ensureOutputDescriptorArray(instruction.output)).flat().map(handleOutputDescriptor);
  return result;
}
/** Determine output files from multiple configs and delete them */

export async function clean(param) {
  const {
    fsx,
    path
  } = param;
  const configLoader = new ConfigLoader(param);
  const errors = [];

  for await (const configFile of param.configFiles) {
    const configResult = await configLoader.loadConfig(configFile);

    if (configResult.code) {
      errors.push(configResult);
      continue;
    }

    const removalResults = cleanUnit({
      fsx,
      path,
      directory: path.dirname(configFile),
      instruction: configResult.value.instruction
    });

    for (const {
      filename,
      promise
    } of removalResults) {
      await promise.catch(error => errors.push(new FileTreeRemovalFailure(filename, error)));
    }
  }

  return MultipleFailures.maybe(errors) || new Success(undefined);
} //# sourceMappingURL=clean.js.map